var sys = require('sys');
var fs = require('fs');
var unittest = require('unittest');

var JSLINT = require('JSLint-commonJS');

var walkTree = function(root, fn) {
    var filenameParts, extension, path, stat;
    var files = fs.readdirSync(root);
    files.forEach(function(file) {
        path = root + '/' + file;
        stat = fs.lstatSync(path);
        fn(path, stat);
        if (stat.isDirectory() && !stat.isSymbolicLink()) {
            walkTree(path, fn);
        }
    });
};

var getFilePaths = function(root) {
    var paths = [];
    walkTree(root, function(file, stat) {
        filenameParts = file.split('.');
        extension = filenameParts[filenameParts.length - 1];
        if (extension == 'js' || extension == 'json') {
            paths.push(file);
        }
    });
    return paths;
};

var root = __dirname + '/../lib';

getLintFreeTestCase = exports.getLintFreeTestCase = function(root) {
    var LintFreeCase = function () { unittest.TestCase.apply(this, arguments); };
    sys.inherits(LintFreeCase, unittest.TestCase);
    
    var paths = getFilePaths(root);
    
    LintFreeCase.prototype.extend({
        isSetUp: false,
        setUp: function () {
            this.assertEqual(this.isSetUp, false);
            
            this.errors = {};
            this.isSetUp = true;
        },
        tearDown: function () {
            this.isSetUp = false;
        },
        testIsLintFree: function () {
            var self = this;
            paths.forEach(function(path) {
                var source = fs.readFileSync(path, 'utf8');
                var lintFree = JSLINT(source);
                if (!lintFree) {
                    self.errors[path] = JSLINT.errors;
                }
            });
            Object.keys(self.errors).forEach(function(path) {
                var errors = self.errors[path];
                errors.forEach(function(error) {
                    self.assertOk(false, "Problem at line " + error.line + " character " + error.character + ": " + error.reason + "\n\t\tFile: " + path);
                });
            });
            this.assertOk(true);
        }
    });
    return LintFreeCase;
};
